from .classes.concrete_classes.anchor import Anchor
from .classes.concrete_classes.binaryrelation import BinaryRelation
from .classes.concrete_classes.class_py import Class
from .classes.concrete_classes.generalization import Generalization
from .classes.concrete_classes.generalizationset import GeneralizationSet
from .classes.concrete_classes.literal import Literal
from .classes.concrete_classes.naryrelation import NaryRelation
from .classes.concrete_classes.note import Note
from .classes.concrete_classes.package import Package
from .classes.concrete_classes.project import Project
from .classes.concrete_classes.property import Property

from .classes.enumerations.aggregationkind import AggregationKind
from .classes.enumerations.classstereotype import ClassStereotype
from .classes.enumerations.ontologicalnature import OntologicalNature
from .classes.enumerations.ontologyrepresentationstyle import OntologyRepresentationStyle
from .classes.enumerations.propertystereotype import PropertyStereotype
from .classes.enumerations.relationstereotype import RelationStereotype

from .classes.datatypes.cardinality import Cardinality
